/* 
   COPYRIGHT 2008 KRUPCZAK.ORG, LLC.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
   USA
 
   For more information, visit:
   http://www.krupczak.org/
*/

/* file - xmpquery.c */
/* rdk@krupczak.org */

#ifdef WIN32
#include <windows.h>
#include <pthread.h>
#endif

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>

#ifndef WIN32
#include <sys/time.h>
#include <unistd.h>
#include <strings.h>
#endif
#include <stdlib.h>

#include <openssl/ssl.h>

#include "xmp.h"
#include "socklib.h"
#include "xmp_session.h"

/* format of versionStr variable is "KorgVersion<space>%3.2f" 
   that way, our software distribution and updating component
   can correctly figure out version of some distributable
   binary thing */   

char *versionStr = "KorgVersion 1.06";

/* added support for SSL and converted xmpquery to use xmp_session
   tools in 1.04 */

/* declare these variables for xmp and xmp_session */

xmpStats_t xmpStats;
int xmp_session_verbose = 0;

/* ************************************************************* */

int main(int argc, char *argv[])
{
     int ret;
     xmpSession_t *sp;
     u_int target;
     int i;
     char *authenUser;
     char *cptr;
     char fqdn[512];
     xmpPdu_t s,r;
     short xmpPort = XMP_PORT;
#ifdef WIN32
     WSADATA wsaData;

     if (WSAStartup(0x202,&wsaData) == SOCKET_ERROR) {
        perror("WSAStartup failed");
        WSACleanup();
        exit(-1);
     }
#endif

     if (argc < 4) {
        fprintf(stderr,"usage: %s host[:port] authenUser queryfile [queryfile]\n",argv[0]);
        exit(-1);
     }

     /* figure out our FQDN */
     get_fqdn(fqdn,511);

     DBG1("xmpquery: using fqdn '%s'\n",fqdn);

     init_ssl_stuff(fqdn,"xmpquery.pem","xmpquery.csr");

     if ((cptr = strchr(argv[1],':')) != NULL) {
        *cptr = 0;
        cptr++;
        xmpPort = atoi(cptr);
     }

     if ((target = hostname_to_ipaddr(argv[1])) == 0) {
        fprintf(stderr,"Unknown host '%s'\n",argv[1]);
        exit(-1);
     }

     authenUser = argv[2];

     if ((sp = newXmpSessionConnect(target,htons(xmpPort),authenUser))== NULL) {
        fprintf(stderr,"Failed to connect to %s\n",argv[1]);
        exit(-1);
     }

     fprintf(stderr,"xmpquery: connected to %s,%d\n",argv[1],xmpPort);
     DBG1("xmpquery: wireHdr_t is %d\n",sizeof(xmpWireHdr_t));

     for (i=3; i<argc; i++) {

         if (parseXmpMessageByFilename(argv[i],&s) > 0) {
	    if (xmpTransaction(sp,&s,&r) > 0) {
	       dumpInternalXmpMessage(&r);               
            }
         }

     }

     destroyXmpSession(sp);
}

/* ************************************************************* */

